from typing import Any, Optional

EMPTY_SHA256_HASH: str
PAYLOAD_BUFFER: Any
ISO8601: str
SIGV4_TIMESTAMP: str
SIGNED_HEADERS_BLACKLIST: Any
UNSIGNED_PAYLOAD: str

class BaseSigner:
    REQUIRES_REGION: bool = ...
    def add_auth(self, request: Any) -> None: ...

class SigV2Auth(BaseSigner):
    credentials: Any = ...
    def __init__(self, credentials: Any) -> None: ...
    def calc_signature(self, request: Any, params: Any) -> Any: ...
    def add_auth(self, request: Any) -> Any: ...

class SigV3Auth(BaseSigner):
    credentials: Any = ...
    def __init__(self, credentials: Any) -> None: ...
    def add_auth(self, request: Any) -> None: ...

class SigV4Auth(BaseSigner):
    REQUIRES_REGION: bool = ...
    credentials: Any = ...
    def __init__(self, credentials: Any, service_name: Any, region_name: Any) -> None: ...
    def headers_to_sign(self, request: Any) -> Any: ...
    def canonical_query_string(self, request: Any) -> Any: ...
    def canonical_headers(self, headers_to_sign: Any) -> Any: ...
    def signed_headers(self, headers_to_sign: Any) -> Any: ...
    def payload(self, request: Any) -> Any: ...
    def canonical_request(self, request: Any) -> Any: ...
    def scope(self, request: Any) -> Any: ...
    def credential_scope(self, request: Any) -> Any: ...
    def string_to_sign(self, request: Any, canonical_request: Any) -> Any: ...
    def signature(self, string_to_sign: Any, request: Any) -> Any: ...
    def add_auth(self, request: Any) -> None: ...

class S3SigV4Auth(SigV4Auth): ...

class SigV4QueryAuth(SigV4Auth):
    DEFAULT_EXPIRES: int = ...
    def __init__(
        self, credentials: Any, service_name: Any, region_name: Any, expires: Any = ...
    ) -> None: ...

class S3SigV4QueryAuth(SigV4QueryAuth):
    def payload(self, request: Any) -> Any: ...

class S3SigV4PostAuth(SigV4Auth):
    def add_auth(self, request: Any) -> None: ...

class HmacV1Auth(BaseSigner):
    QSAOfInterest: Any = ...
    credentials: Any = ...
    def __init__(
        self,
        credentials: Any,
        service_name: Optional[Any] = ...,
        region_name: Optional[str] = ...,
    ) -> None: ...
    def sign_string(self, string_to_sign: Any) -> Any: ...
    def canonical_standard_headers(self, headers: Any) -> Any: ...
    def canonical_custom_headers(self, headers: Any) -> Any: ...
    def unquote_v(self, nv: Any) -> Any: ...
    def canonical_resource(self, split: Any, auth_path: Optional[Any] = ...) -> Any: ...
    def canonical_string(
        self,
        method: Any,
        split: Any,
        headers: Any,
        expires: Optional[Any] = ...,
        auth_path: Optional[Any] = ...,
    ) -> Any: ...
    def get_signature(
        self,
        method: Any,
        split: Any,
        headers: Any,
        expires: Optional[Any] = ...,
        auth_path: Optional[Any] = ...,
    ) -> Any: ...
    def add_auth(self, request: Any) -> None: ...

class HmacV1QueryAuth(HmacV1Auth):
    DEFAULT_EXPIRES: int = ...
    credentials: Any = ...
    def __init__(self, credentials: Any, expires: Any = ...) -> None: ...

class HmacV1PostAuth(HmacV1Auth):
    def add_auth(self, request: Any) -> None: ...

AUTH_TYPE_MAPS: Any



from typing import Any, Optional

from botocore.exceptions import NoRegionError as NoRegionError

DEFAULT_URI_TEMPLATE: str
DEFAULT_SERVICE_DATA: Any

class BaseEndpointResolver:
    def construct_endpoint(self, service_name: Any, region_name: Optional[str] = ...) -> None: ...
    def get_available_partitions(self) -> None: ...
    def get_available_endpoints(
        self,
        service_name: Any,
        partition_name: str = ...,
        allow_non_regional: bool = ...,
    ) -> None: ...

class EndpointResolver(BaseEndpointResolver):
    def __init__(self, endpoint_data: Any) -> None: ...
    def get_available_partitions(self) -> Any: ...
    def get_available_endpoints(
        self,
        service_name: Any,
        partition_name: str = ...,
        allow_non_regional: bool = ...,
    ) -> Any: ...
    def construct_endpoint(
        self,
        service_name: Any,
        region_name: Optional[str] = ...,
        partition_name: Optional[Any] = ...,
    ) -> Any: ...



from typing import Dict, Tuple, Union

from botocore.compat import OrderedDict as OrderedDict
from botocore.endpoint import DEFAULT_TIMEOUT as DEFAULT_TIMEOUT
from botocore.endpoint import MAX_POOL_CONNECTIONS as MAX_POOL_CONNECTIONS
from botocore.exceptions import InvalidMaxRetryAttemptsError as InvalidMaxRetryAttemptsError
from botocore.exceptions import InvalidRetryConfigurationError as InvalidRetryConfigurationError
from botocore.exceptions import InvalidRetryModeError as InvalidRetryModeError
from botocore.exceptions import InvalidS3AddressingStyleError as InvalidS3AddressingStyleError
from typing_extensions import Literal, TypedDict

class _RetryDict(TypedDict, total=False):
    total_max_attempts: int
    max_attempts: int
    mode: Literal["legacy", "standard", "adaptive"]

class _S3Dict(TypedDict, total=False):
    use_accelerate_endpoint: bool
    payload_signing_enabled: bool
    addressing_style: Literal["auto", "virtual", "path"]
    us_east_1_regional_endpoint: Literal["regional", "legacy"]

class Config:
    OPTION_DEFAULTS: OrderedDict[str, None]
    def __init__(
        self,
        region_name: str = ...,
        signature_version: str = ...,
        user_agent: str = ...,
        user_agent_extra: str = ...,
        connect_timeout: Union[float, int] = ...,
        read_timeout: Union[float, int] = ...,
        parameter_validation: bool = ...,
        max_pool_connections: int = ...,
        proxies: Dict[str, str] = ...,
        s3: _S3Dict = ...,
        retries: _RetryDict = ...,
        client_cert: Union[str, Tuple[str, str]] = ...,
        inject_host_prefix: bool = ...,
    ) -> None: ...
    def merge(self, other_config: Config) -> Config: ...



from typing import Any, Optional

from botocore.compat import ETree as ETree
from botocore.compat import XMLParseError as XMLParseError
from botocore.eventstream import EventStream as EventStream
from botocore.eventstream import NoInitialResponseError as NoInitialResponseError
from botocore.utils import is_json_value_header as is_json_value_header
from botocore.utils import lowercase_dict as lowercase_dict
from botocore.utils import merge_dicts as merge_dicts
from botocore.utils import parse_timestamp as parse_timestamp

DEFAULT_TIMESTAMP_PARSER = parse_timestamp

class ResponseParserFactory:
    def __init__(self) -> None: ...
    def set_parser_defaults(self, **kwargs: Any) -> None: ...
    def create_parser(self, protocol_name: Any) -> Any: ...

def create_parser(protocol: Any) -> Any: ...

class ResponseParserError(Exception): ...

class ResponseParser:
    DEFAULT_ENCODING: str = ...
    EVENT_STREAM_PARSER_CLS: Any = ...
    def __init__(
        self, timestamp_parser: Optional[Any] = ..., blob_parser: Optional[Any] = ...
    ) -> None: ...
    def parse(self, response: Any, shape: Any) -> Any: ...

class BaseXMLResponseParser(ResponseParser):
    def __init__(
        self, timestamp_parser: Optional[Any] = ..., blob_parser: Optional[Any] = ...
    ) -> None: ...

class QueryParser(BaseXMLResponseParser): ...
class EC2QueryParser(QueryParser): ...
class BaseJSONParser(ResponseParser): ...
class BaseEventStreamParser(ResponseParser): ...
class EventStreamJSONParser(BaseEventStreamParser, BaseJSONParser): ...
class EventStreamXMLParser(BaseEventStreamParser, BaseXMLResponseParser): ...

class JSONParser(BaseJSONParser):
    EVENT_STREAM_PARSER_CLS: Any = ...

class BaseRestParser(ResponseParser): ...

class RestJSONParser(BaseRestParser, BaseJSONParser):
    EVENT_STREAM_PARSER_CLS: Any = ...

class RestXMLParser(BaseRestParser, BaseXMLResponseParser):
    EVENT_STREAM_PARSER_CLS: Any = ...

PROTOCOL_PARSERS: Any



from typing import Any

from botocore.exceptions import ClientError as ClientError
from botocore.utils import get_service_module_name as get_service_module_name

class BaseClientExceptions:
    ClientError: Any = ...
    def __init__(self, code_to_exception: Any) -> None: ...
    def from_code(self, error_code: Any) -> Any: ...
    def __getattr__(self, name: Any) -> None: ...

class ClientExceptionsFactory:
    def __init__(self) -> None: ...
    def create_client_exceptions(self, service_model: Any) -> Any: ...



from typing import Any, Dict, List, Optional, Union

from botocore import UNSIGNED as UNSIGNED
from botocore import waiter as waiter
from botocore import xform_name as xform_name
from botocore.args import ClientArgsCreator as ClientArgsCreator
from botocore.auth import AUTH_TYPE_MAPS as AUTH_TYPE_MAPS
from botocore.awsrequest import prepare_request_dict as prepare_request_dict
from botocore.config import Config as Config
from botocore.discovery import EndpointDiscoveryHandler as EndpointDiscoveryHandler
from botocore.discovery import EndpointDiscoveryManager as EndpointDiscoveryManager
from botocore.discovery import (
    block_endpoint_discovery_required_operations as block_endpoint_discovery_required_operations,
)
from botocore.exceptions import ClientError as ClientError
from botocore.exceptions import DataNotFoundError as DataNotFoundError
from botocore.exceptions import (
    InvalidEndpointDiscoveryConfigurationError as InvalidEndpointDiscoveryConfigurationError,
)
from botocore.exceptions import OperationNotPageableError as OperationNotPageableError
from botocore.exceptions import UnknownSignatureVersionError as UnknownSignatureVersionError
from botocore.history import get_global_history_recorder as get_global_history_recorder
from botocore.hooks import first_non_none_response as first_non_none_response
from botocore.loaders import Loader
from botocore.model import ServiceModel as ServiceModel
from botocore.paginate import Paginator as Paginator
from botocore.retries import adaptive as adaptive
from botocore.retries import standard as standard
from botocore.utils import CachedProperty as CachedProperty
from botocore.utils import S3ArnParamHandler as S3ArnParamHandler
from botocore.utils import S3ControlArnParamHandler as S3ControlArnParamHandler
from botocore.utils import S3ControlEndpointSetter as S3ControlEndpointSetter
from botocore.utils import S3EndpointSetter as S3EndpointSetter
from botocore.utils import S3RegionRedirector as S3RegionRedirector
from botocore.utils import ensure_boolean as ensure_boolean
from botocore.utils import get_service_module_name as get_service_module_name

class ClientCreator:
    def __init__(
        self,
        loader: Loader,
        endpoint_resolver: Any,
        user_agent: str,
        event_emitter: Any,
        retry_handler_factory: Any,
        retry_config_translator: Any,
        response_parser_factory: Optional[Any] = ...,
        exceptions_factory: Optional[Any] = ...,
        config_store: Optional[Any] = ...,
    ) -> None: ...
    def create_client(
        self,
        service_name: str,
        region_name: str,
        is_secure: bool = ...,
        endpoint_url: Optional[str] = ...,
        verify: Optional[Union[str, bool]] = ...,
        credentials: Optional[Any] = ...,
        scoped_config: Optional[Any] = ...,
        api_version: Optional[str] = ...,
        client_config: Optional[Config] = ...,
    ) -> None: ...
    def create_client_class(self, service_name: str, api_version: Optional[Any] = ...) -> None: ...

class ClientEndpointBridge:
    DEFAULT_ENDPOINT: str = ...
    service_signing_name: str = ...
    endpoint_resolver: Any = ...
    scoped_config: Any = ...
    client_config: Any = ...
    default_endpoint: Any = ...
    def __init__(
        self,
        endpoint_resolver: Any,
        scoped_config: Optional[Any] = ...,
        client_config: Optional[Any] = ...,
        default_endpoint: Optional[str] = ...,
        service_signing_name: Optional[Any] = ...,
    ) -> None: ...
    def resolve(
        self,
        service_name: Any,
        region_name: Optional[str] = ...,
        endpoint_url: Optional[str] = ...,
        is_secure: bool = ...,
    ) -> None: ...

class BaseClient:
    meta: ClientMeta
    def __init__(
        self,
        serializer: Any,
        endpoint: str,
        response_parser: Any,
        event_emitter: Any,
        request_signer: Any,
        service_model: Any,
        loader: Any,
        client_config: Config,
        partition: Any,
        exceptions_factory: Any,
    ) -> None: ...
    def __getattr__(self, item: str) -> Any: ...
    def get_paginator(self, operation_name: str) -> Paginator: ...
    def can_paginate(self, operation_name: str) -> bool: ...
    def get_waiter(self, waiter_name: str) -> waiter.Waiter: ...
    @property
    def waiter_names(self) -> List[str]: ...
    @property
    def exceptions(self) -> Any: ...

class ClientMeta:
    events: Any = ...
    def __init__(
        self,
        events: Any,
        client_config: Config,
        endpoint_url: str,
        service_model: Any,
        method_to_api_mapping: Dict[str, str],
        partition: Any,
    ) -> None: ...
    @property
    def service_model(self) -> ServiceModel: ...
    @property
    def region_name(self) -> str: ...
    @property
    def endpoint_url(self) -> str: ...
    @property
    def config(self) -> Any: ...
    @property
    def method_to_api_mapping(self) -> Dict[str, str]: ...
    @property
    def partition(self) -> str: ...
